//
// CommandLineParserTests.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.application;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import java.util.ArrayList;
import java.util.Collection;

import org.junit.Before;
import org.junit.Test;

import com.dakengo.libs.application.CommandLineParser;

/**
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class CommandLineParserTests
{
   @Before
   public void setUp()
   {
      m_parser = new CommandLineParser();
   }

   @Test
   public void testParser1()
   {
      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS3 );
      assertEmptyArgs();
      remainder.add( ARGS3[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS4 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS4[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS5 );
      assertEmptyArgs();
      remainder.add( ARGS5[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS6 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS6[0] );
      remainder.add( ARGS6[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS9 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS9[0] );
      remainder.add( ARGS9[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS13 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS13[0] );
      remainder.add( ARGS13[1] );
      remainder.add( ARGS13[2] );
      remainder.add( ARGS13[3] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser2()
   {
      m_parser.addCommand( FOO_CMD );

      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS3 );
      assertEmptyArgs();
      remainder.add( ARGS3[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS4 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS4[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS5 );
      assertEmptyArgs();
      remainder.add( ARGS5[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS6 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS6[0] );
      remainder.add( ARGS6[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS7 );
      assertArgs( FOO_CMD, '\0', "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS9 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS9[0] );
      remainder.add( ARGS9[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS13 );
      assertArgs( FOO_CMD, '\0', "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      remainder.clear();
      remainder.add( ARGS13[1] );
      remainder.add( ARGS13[2] );
      remainder.add( ARGS13[3] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser3()
   {
      m_parser.addOption( A_OPT, true );

      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS4 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS4[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS8 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS11 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS11[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS14 );
      assertArgs( "", A_OPT, ARGS14[2], '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      remainder.clear();
      remainder.add( ARGS14[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS15 );
      assertArgs( "", A_OPT, ARGS15[1], '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser4()
   {
      m_parser.addOption( A_OPT, false );

      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS4 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS4[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS8 );
      assertArgs( "", A_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS11 );
      assertArgs( "", A_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      remainder.clear();
      remainder.add( ARGS11[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS14 );
      assertArgs( "", A_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      remainder.clear();
      remainder.add( ARGS14[0] );
      remainder.add( ARGS14[2] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS15 );
      assertArgs( "", A_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      remainder.clear();
      remainder.add( ARGS15[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser5()
   {
      m_parser.addOption( A_OPT, true );
      m_parser.addOption( B_OPT, false );

      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS5 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS5[0] );
      remainder.add( ARGS5[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS8 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS9 );
      assertArgs( "", B_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      remainder.clear();
      remainder.add( ARGS9[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS11 );
      assertArgs( "", B_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS15 );
      assertArgs( "", A_OPT, ARGS15[1], '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser6()
   {
      m_parser.addCommand( FOO_CMD );
      m_parser.addCommand( BAR_CMD );
      m_parser.addOption( A_OPT, true );
      m_parser.addOption( B_OPT, false );

      m_parser.process( ARGS1 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS2 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS2[0] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS5 );
      assertEmptyArgs();
      remainder.clear();
      remainder.add( ARGS5[0] );
      remainder.add( ARGS5[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS7 );
      assertArgs( FOO_CMD, '\0', "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS8 );
      assertEmptyArgs();
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS9 );
      assertArgs( "", B_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      remainder.clear();
      remainder.add( ARGS9[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS12 );
      assertArgs( FOO_CMD, B_OPT, "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$
      remainder.clear();
      remainder.add( ARGS12[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertValueMessage( A_OPT );

      m_parser.process( ARGS14 );
      assertArgs( FOO_CMD, A_OPT, ARGS14[2], '\0', "" ); //$NON-NLS-1$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS16 );
      assertArgs( BAR_CMD, B_OPT, "", A_OPT, ARGS16[4] ); //$NON-NLS-1$
      remainder.clear();
      remainder.add( ARGS16[2] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   @Test
   public void testParser7()
   {
      m_parser.addOption( A_OPT, false );
      m_parser.addOption( B_OPT, false );

      m_parser.process( ARGS5 );
      assertEmptyArgs();
      Collection<String> remainder = new ArrayList<String>();
      remainder.add( ARGS5[0] );
      remainder.add( ARGS5[1] );
      assertThat( m_parser.getRemainingArgs(), is( remainder ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );

      m_parser.process( ARGS11 );
      assertArgs( "", A_OPT, "", B_OPT, "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
      assertThat( m_parser.getRemainingArgs().isEmpty(), is( true ) );
      assertThat( m_parser.getMessages().isEmpty(), is( true ) );
   }

   private void assertEmptyArgs()
   {
      assertArgs( "", '\0', "", '\0', "" ); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
   }

   private void assertArgs( String cmd, char opt1_char, String opt1_val, char opt2_char, String opt2_val )
   {
      if (cmd.isEmpty())
         assertThat( m_parser.getCommand().isEmpty(), is( true ) );
      else
         assertThat( m_parser.getCommand(), is( cmd ) );
      if (opt1_char != '\0')
      {
         if (opt1_val.isEmpty())
         {
            assertThat( m_parser.optionRequiresValue( opt1_char ), is( false ) );
         }
         else
         {
            assertThat( m_parser.optionRequiresValue( opt1_char ), is( true ) );
            assertThat( m_parser.optionSet( opt1_char ), is( true ) );
            assertThat( m_parser.getOptionValue( opt1_char ), is( opt1_val ) );
         }
      }
      if (opt2_char != '\0')
      {
         if (opt2_val.isEmpty())
         {
            assertThat( m_parser.optionRequiresValue( opt2_char ), is( false ) );
         }
         else
         {
            assertThat( m_parser.optionRequiresValue( opt2_char ), is( true ) );
            assertThat( m_parser.optionSet( opt2_char ), is( true ) );
            assertThat( m_parser.getOptionValue( opt2_char ), is( opt2_val ) );
         }
      }
   }

   private void assertValueMessage( char opt_char )
   {
      boolean found = false;
      String expected = "option -" + opt_char + " requires a value";
      for (String msg : m_parser.getMessages())
      {
         if (msg.equals( expected ))
         {
            found = true;
            break;
         }
      }
      assertThat( found, is( true ) );
   }

   private static final String  FOO_CMD  = "foo"; //$NON-NLS-1$
   private static final String  BAR_CMD  = "bar"; //$NON-NLS-1$
   private static final char    A_OPT    = 'a';
   private static final char    B_OPT    = 'b';

   private static final String[]  ARGS1  = new String[] {};
   private static final String[]  ARGS2  = new String[] { "xxx" };         //$NON-NLS-1$
   private static final String[]  ARGS3  = new String[] { "xxx", "yyy" };  //$NON-NLS-1$ //$NON-NLS-2$
   private static final String[]  ARGS4  = new String[] { "-f" };          //$NON-NLS-1$
   private static final String[]  ARGS5  = new String[] { "-f", "-g" };    //$NON-NLS-1$ //$NON-NLS-2$
   private static final String[]  ARGS6  = new String[] { "xxx", "-f" };   //$NON-NLS-1$ //$NON-NLS-2$
   private static final String[]  ARGS7  = new String[] { "foo" };         //$NON-NLS-1$
   private static final String[]  ARGS8  = new String[] { "-a" };          //$NON-NLS-1$
   private static final String[]  ARGS9  = new String[] { "-b", "bar" };   //$NON-NLS-1$ //$NON-NLS-2$
/*
   private static final String[]  ARGS10 = new String[] { "foo", "bar" };  //$NON-NLS-1$ //$NON-NLS-2$
*/
   private static final String[]  ARGS11 = new String[] { "-a", "-b" };                      //$NON-NLS-1$ //$NON-NLS-2$
   private static final String[]  ARGS12 = new String[] { "foo", "bar", "-a", "-b" };        //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
   private static final String[]  ARGS13 = new String[] { "foo", "xxx", "-a", "-f" };        //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
   private static final String[]  ARGS14 = new String[] { "foo", "-a", "abc" };              //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
   private static final String[]  ARGS15 = new String[] { "-a", "foo" };                     //$NON-NLS-1$ //$NON-NLS-2$
   private static final String[]  ARGS16 = new String[] { "bar", "-b", "foo", "-a", "abc" }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$

   private CommandLineParser m_parser;
}
