//
// TextUtilsTests.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.application;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;

import org.junit.Test;

import com.dakengo.libs.application.Resources;
import com.dakengo.libs.application.TextUtils;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;

public class TextUtilsTests
{
   @Test
   public void testPrintBoard() throws IOException
   {
      Board.setSize( 3 );
      Board board = new Board();
      board.setColor( (short) 0, Colors.BLACK );
      board.setColor( (short) 4, Colors.WHITE );
      board.setColor( (short) 8, Colors.BLACK );

      StringWriter buffer = new StringWriter();
      BufferedWriter bw = new BufferedWriter( buffer );
      bw.append( "3   . . X " ); //$NON-NLS-1$
      bw.newLine();
      bw.append( "2   . O . " ); //$NON-NLS-1$
      bw.newLine();
      bw.append( "1   X . . " ); //$NON-NLS-1$
      bw.newLine();
      bw.newLine();
      bw.append( "    A B C" ); //$NON-NLS-1$
      bw.flush();

      assertThat( TextUtils.print( board ), is( buffer.toString() ) );
   }

   @Test
   public void testArrayToString()
   {
      Integer[] values = new Integer[] { 0, 1, 2 };
      assertThat( TextUtils.arrayToString( values, " " ), is( "0 1 2" ) );                                  //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( TextUtils.arrayToString( values, "-+-" ), is( "0-+-1-+-2" ) );                            //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( TextUtils.arrayToString( new Integer[] {}, " " ), is( Resources.getString( "msg.1" ) ) ); //$NON-NLS-1$
   }

   @Test
   public void testCollectionToString()
   {
      ArrayList<Integer> values = new ArrayList<Integer>();
      assertThat( TextUtils.collectionToString( values, " ", false, null ), is( Resources.getString( "msg.2" ) ) ); //$NON-NLS-1$
      values.add( 2 );
      values.add( 0 );
      values.add( 1 );
      assertThat( TextUtils.collectionToString( values, " ", false, null ), is( "2 0 1" ) );                        //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( TextUtils.collectionToString( values, "-+-", false, null ), is( "2-+-0-+-1" ) );                  //$NON-NLS-1$ //$NON-NLS-2$
      assertThat( TextUtils.collectionToString( values, " ", true, null ), is( "0 1 2" ) );                         //$NON-NLS-1$ //$NON-NLS-2$
   }
}
