//
// PlayoutTests.java
//
// Copyright (c) 2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.player;

import java.io.IOException;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import org.apache.commons.lang3.tuple.ImmutablePair;
import org.junit.Before;
import org.junit.Test;

import com.dakengo.libs.application.Configuration;
import com.dakengo.libs.application.PluginUtils;
import com.dakengo.libs.base.MersenneTwisterFast;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.game.Scorer;
import com.dakengo.libs.player.DefaultPlayout;
import com.dakengo.libs.player.Playout;
import com.dakengo.tests.utils.AssertUtils;
import com.dakengo.tests.utils.BaseConfiguredTests;

public class PlayoutTests extends BaseConfiguredTests
{
   public PlayoutTests() throws IOException
   {
      m_prng = new MersenneTwisterFast();
      m_scorer = m_plugin_mgr.getPlugin( Scorer.class, PluginUtils.getScoringCapability( Configuration.DEFAULT_SCORING_SYSTEM ) );
   }

   @Before
   public void setUp()
   {
      Board.setSize( 3 );
      m_board = new Board();
      for (int pnt = 0; pnt < 9; pnt++)
         m_board.setColor( (short) pnt, Colors.BLACK );
      m_playout = new DefaultPlayout( m_prng, m_scorer );
   }

   @Test
   public void testPlayouts()
   {
      Board orig = new Board( m_board );
      AssertUtils.assertThrows(
         new Runnable() { @Override public void run() { m_playout.run( m_board, (short) 2, Colors.BLACK, 1 ); } },
         IllegalArgumentException.class, null );
      assertThat( m_board, is( orig ) );

      m_board.setColor( (short) 4, Colors.EMPTY );
      orig.setColor( (short) 4, Colors.EMPTY );
      AssertUtils.assertThrows(
         new Runnable() { @Override public void run() { m_playout.run( m_board, (short) 4, Colors.BLACK, 1 ); } },
         IllegalStateException.class, null );

      m_board.setColor( (short) 3, Colors.EMPTY );
      orig.setColor( (short) 3, Colors.EMPTY );
      ImmutablePair<Integer,Long> result = m_playout.run( m_board, (short) 4, Colors.BLACK, 1 );
      assertThat( m_board, is( orig ) );
      assertThat( result.getLeft(), is( 1 ) );
      assertThat( result.getRight(), greaterThanOrEqualTo( 0L ) );

      result = m_playout.run( m_board, (short) 4, Colors.WHITE, 1 );
      assertThat( m_board, is( orig ) );
      assertThat( result.getLeft(), is( 1 ) );
      assertThat( result.getRight(), greaterThanOrEqualTo( 0L ) );

      result = m_playout.run( m_board, (short) 4, Colors.BLACK, 10 );
      assertThat( m_board, is( orig ) );
      assertThat( result.getLeft(), is( 1 ) );
      assertThat( result.getRight(), greaterThanOrEqualTo( 0L ) );

      result = m_playout.run( m_board, (short) 4, Colors.WHITE, 10 );
      assertThat( m_board, is( orig ) );
      assertThat( result.getLeft(), is( 1 ) );
      assertThat( result.getRight(), greaterThanOrEqualTo( 0L ) );

      m_board.setColor( (short) 5, Colors.EMPTY );
      orig.setColor( (short) 5, Colors.EMPTY );
      result = m_playout.run( m_board, (short) 3, Colors.BLACK, 500 );
      assertThat( result.getLeft(), is( -500 ) );
      assertThat( result.getRight(), greaterThanOrEqualTo( 0L ) );
   }

   private Board               m_board;
   private Playout             m_playout;
   private Scorer              m_scorer;
   private MersenneTwisterFast m_prng;
}
