//
// TestPlayer.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.plugins.player;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.google.gson.JsonElement;
import net.xeoh.plugins.base.annotations.Capabilities;
import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.Constants;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Game;
import com.dakengo.libs.game.MoveTypes;
import com.dakengo.libs.player.Player;

@PluginImplementation
public class TestPlayer implements Player
{
   @Override
   public void init( GameContext context, Map<String,JsonElement> unused )
   {
      m_game_id = context.getGameID();
      m_turn = 0;
   }

   @Override
   public short getMove( Game game )
   {
      if (m_turn < 0)
         throw new IllegalStateException( "test player not initialized" );
      m_turn++;
      short point = game.getLastPoint();
      if (point == Board.INVALID_POINT)
         point = (short) 0;
      else if (game.getLastMoveType() == MoveTypes.PASS || point >= 10)
         point = Board.INVALID_POINT;
      else
         point++;
      return point;
   }

   @Override
   public Set<Short> getHandicap( int count )
   {
      if (m_turn < 0)
         throw new IllegalStateException( "test player not initialized" );
      m_turn++;
      Set<Short> points = new HashSet<Short>();
      while (count > 0)
      {
         points.add( (short) (Board.getTotal() - count) );
         count--;
      }
      return points;
   }

   @Override
   public void reset( String game_id )
   {
      if (!m_game_id.equals( game_id ))
         throw new IllegalArgumentException( String.format( "game_id argument '%s' does not match value passed to init() '%s'", game_id, m_game_id ) );
      if (m_turn < 0)
         throw new IllegalStateException( "test player not initialized" );
      m_turn = 0;
   }

   @Override
   public String getName( String unused1, String unused2 )
   {
      return "Test Player";
   }

   @Capabilities
   public String[] getCapabilities()
   {
      return CAPABILITIES;
   }

   private static String[] CAPABILITIES;

   static
   {
      CAPABILITIES = new String[1];
      CAPABILITIES[0] = Constants.PLAYER_CAP_PREFIX + "test"; //$NON-NLS-1$
   }

   private String m_game_id = ""; //$NON-NLS-1$
   private int    m_turn    = -1;
}
