//
// BaseConfiguredTests.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.utils;

import static org.junit.Assert.*;

import java.io.File;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Properties;

import net.xeoh.plugins.base.PluginManager;

import com.dakengo.libs.application.ConfigReader;
import com.dakengo.libs.application.Configuration;
import com.dakengo.libs.application.PluginUtils;

/**
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public abstract class BaseConfiguredTests
{
   protected BaseConfiguredTests() throws IOException
   {
      if (m_config == null)
      {
         try (InputStreamReader input = new InputStreamReader( getClass().getResourceAsStream( "/data/test-config.json" ) )) //$NON-NLS-1$
         {
            m_config = ConfigReader.read( input );
         }
         if (Configuration.class.getPackage().getImplementationVersion() == null)
         {
            File proj_dir = new File( System.getProperty( "dakengo.app.home.dir" ) ); //$NON-NLS-1$
            File props_file = new File( proj_dir, "version.properties" ); //$NON-NLS-1$
            assertTrue( props_file.exists() );
            try (FileReader reader = new FileReader( props_file ))
            {
               Properties props = new Properties();
               props.load( reader );
               String version = props.getProperty( "version" ); //$NON-NLS-1$
               assertTrue( version != null && !version.isEmpty() );
               m_config.setEngineVersion( version );
            }
         }
      }
      if (m_plugin_mgr == null)
         m_plugin_mgr = PluginUtils.createManager( m_config );
   }

   protected static PluginManager m_plugin_mgr;
   protected static Configuration m_config;
}
