//
// BaseRecordTests.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.utils;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.dakengo.libs.application.PluginUtils;
import com.dakengo.libs.record.BoardCollector;
import com.dakengo.libs.record.Record;
import com.dakengo.libs.record.RecordReader;
import com.dakengo.libs.record.RecordWriter;

/**
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public abstract class BaseRecordTests extends BaseConfiguredTests
{
   protected BaseRecordTests() throws IOException
   {
   }

   protected void initReader( String file_ext )
   {
      assertFalse( file_ext.isEmpty() );
      if (m_reader_file_ext.equalsIgnoreCase( file_ext ))
         return;
      m_reader_file_ext = file_ext;
      m_reader = m_plugin_mgr.getPlugin( RecordReader.class, PluginUtils.getRecordCapabilities( m_reader_file_ext, true ) );
      assertNotNull( m_reader );
   }

   protected void initWriter( String file_ext )
   {
      assertFalse( file_ext.isEmpty() );
      if (m_writer_file_ext.equalsIgnoreCase( file_ext ))
         return;
      m_writer_file_ext = file_ext;
      m_writer = m_plugin_mgr.getPlugin( RecordWriter.class, PluginUtils.getRecordCapabilities( m_writer_file_ext, false ) );
      assertNotNull( m_writer );
   }

   protected Record load( String filename, int expected_msg_count ) throws IOException
   {
      Record rec = null;
      m_reader.setInputPath( filename );
      try (BufferedReader br = new BufferedReader( new InputStreamReader( getClass().getResourceAsStream( "/data/" + filename ) ) )) //$NON-NLS-1$
      {
         m_reader.setInput( br );
         rec = m_reader.read();
         if (expected_msg_count >= 0)
            assertParseMessageCount( expected_msg_count );
         assertNull( m_reader.read() );
      }
      return rec;
   }

   protected List<Record> load( String filename, int... expected_msg_counts ) throws IOException
   {
      List<Record> records = new ArrayList<Record>();
      m_reader.setInputPath( filename );
      try (BufferedReader br = new BufferedReader( new InputStreamReader( getClass().getResourceAsStream( "/data/" + filename ) ) )) //$NON-NLS-1$
      {
         m_reader.setInput( br );
         Record rec = null;
         int i = 0;
         while ((rec = m_reader.read()) != null)
         {
            if (expected_msg_counts[i] >= 0)
               assertParseMessageCount( expected_msg_counts[i] );
            records.add( rec );
            i++;
         }
      }
      return records;
   }

   protected void assertParseMessageCount( int msg_count )
   {
      String sep = System.getProperty( "line.separator" ); //$NON-NLS-1$
      StringBuilder buffer = new StringBuilder();
      for (String msg : m_reader.getMessages())
      {
         buffer.append( msg );
         buffer.append( sep ); 
      }
      assertThat( buffer.toString(), m_reader.getMessages().size(), is( msg_count ) );
   }

   protected String            m_reader_file_ext = ""; //$NON-NLS-1$
   protected String            m_writer_file_ext = ""; //$NON-NLS-1$
   protected TestPropVisitor   m_prop_visitor    = new TestPropVisitor( true );
   protected BoardCollector    m_collector       = new BoardCollector();
   protected RecordReader      m_reader;
   protected RecordWriter      m_writer;
}
