//
// DkAssertUtils.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.utils;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import java.util.Set;

import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;

/**
 * This class provides Dakengo-specific assertion utility methods for
 * implementing unit tests.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack Unrue</a>
 */
public abstract class DkAssertUtils
{
   /**
    * This method asserts that the specified points are present on the board.
    * Any points not present in {@code black_pnts} or {@code white_pnts} are
    * asserted to be empty points.
    * 
    * @param board
    * @param black_pnts  can be empty or null
    * @param white_pnts  can be empty or null
    */
   public static void assertBoardPoints( Board board, Set<Short> black_pnts, Set<Short> white_pnts )
   {
      int empty_count = Board.getTotal(); 
      if (black_pnts != null)
      {
         assertThat( "black count", board.getCount( Colors.BLACK ), is( black_pnts.size() ) ); //$NON-NLS-1$
         empty_count -= black_pnts.size();
         assertThat( empty_count, greaterThanOrEqualTo( 0 ) );
         for (short pnt : black_pnts)
            assertThat( "black point", board.getColor( pnt ), is( Colors.BLACK ) ); //$NON-NLS-1$
      }
      if (white_pnts != null)
      {
         assertThat( "white count", board.getCount( Colors.WHITE ), is( white_pnts.size() ) ); //$NON-NLS-1$
         empty_count -= white_pnts.size();
         assertThat( empty_count, greaterThanOrEqualTo( 0 ) );
         for (short pnt : white_pnts)
            assertThat( "white point", board.getColor( pnt ), is( Colors.WHITE ) ); //$NON-NLS-1$
      }
      assertThat( "empty points", board.getCount( Colors.EMPTY ), is( empty_count ) ); //$NON-NLS-1$
   }

   /**
    * This method asserts that the specified points are present on the board.
    * Any points not present in {@code black_pnts} or {@code white_pnts} are
    * asserted to be empty points.
    * 
    * @param board
    * @param black_pnts  can be empty or null
    * @param white_pnts  can be empty or null
    */
   public static void assertBoardPoints( Board board, short[] black_pnts, short[] white_pnts )
   {
      int empty_count = Board.getTotal(); 
      if (black_pnts != null)
      {
         assertThat( "black count", board.getCount( Colors.BLACK ), is( black_pnts.length ) ); //$NON-NLS-1$
         empty_count -= black_pnts.length;
         assertThat( empty_count, greaterThanOrEqualTo( 0 ) );
         for (short pnt : black_pnts)
            assertThat( "black point", board.getColor( pnt ), is( Colors.BLACK ) ); //$NON-NLS-1$
      }
      if (white_pnts != null)
      {
         assertThat( "white count", board.getCount( Colors.WHITE ), is( white_pnts.length ) ); //$NON-NLS-1$
         empty_count -= white_pnts.length;
         assertThat( empty_count, greaterThanOrEqualTo( 0 ) );
         for (short pnt : white_pnts)
            assertThat( "white point", board.getColor( pnt ), is( Colors.WHITE ) ); //$NON-NLS-1$
      }
      assertThat( "empty points", board.getCount( Colors.EMPTY ), is( empty_count ) ); //$NON-NLS-1$
   }
}
