//
// GtpRecordValidator.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.utils;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import java.io.IOException;

import com.dakengo.libs.application.GtpRecordStore;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.record.GameProperties;
import com.dakengo.libs.record.Record;

public class GtpRecordValidator implements GtpRecordStore
{
   public void validate()
   {
      assertNotNull( "save() method not called", m_actual );

      GameProperties props = m_actual.getGameProps();
      assertThat( "black", props.getPlayerName( Colors.BLACK ), is( m_black_player ) );
      assertThat( "white", props.getPlayerName( Colors.WHITE ), is( m_white_player ) );
      assertThat( "size", props.getSize(), is( m_size ) );
   }

   public void setExpectedGameProps( String black, String white, int size )
   {
      assertTrue( Board.isValidSize( size ) );
      m_actual = null;
      m_black_player = black;
      m_white_player = white;
      m_size = size;
   }

   @Override
   public void save( Record rec ) throws IOException
   {
      m_actual = rec;
   }

   private Record m_actual;
   private String m_black_player = "";
   private String m_white_player = "";
   private int    m_size         = -1;
}
