//
// GtpReplyValidator.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.tests.utils;

import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GtpReplyHandler;

public class GtpReplyValidator implements GtpReplyHandler
{
   public void clearInvoked()
   {
      m_invoked = false;
   }

   public boolean wasInvoked()
   {
      return m_invoked;
   }

   public void setAnyTextOK( boolean exp_error )
   {
      m_any_ok = true;
      m_expected_error = exp_error;
   }

   public void setExpected( String cmd_name, boolean exp_error, long exp_id, String exp_text )
   {
      m_cmd_name = cmd_name;
      m_expected_error = exp_error;
      m_expected_id = exp_id;
      m_expected_text = exp_text;
      m_any_ok = false;
      m_invoked = false;
   }

   @Override
   public void handleReply( boolean error, long id, String text )
   {
      m_invoked = true;
      assertThat( m_cmd_name + " command error: " + text, error, is( m_expected_error ) ); //$NON-NLS-1$
      if (m_any_ok)
         return;
      assertThat( m_cmd_name + " command ID", id, is( m_expected_id ) ); //$NON-NLS-1$
      assertThat( m_cmd_name + " command text", text, is( m_expected_text ) ); //$NON-NLS-1$
   }

   private String  m_cmd_name       = ""; //$NON-NLS-1$
   private boolean m_expected_error = false;
   private long    m_expected_id    = GtpCommand.NO_ID;
   private String  m_expected_text  = ""; //$NON-NLS-1$
   private boolean m_any_ok         = false;
   private boolean m_invoked        = false;
}
