//
// FileStore.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.apps.bot;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import com.dakengo.libs.application.Configuration;
import com.dakengo.libs.application.GtpRecordStore;
import com.dakengo.libs.base.Constants;
import com.dakengo.libs.record.Record;
import com.dakengo.libs.record.RecordWriter;

/**
 * This class stores game records in a file.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
class FileStore implements GtpRecordStore
{
   @Override
   public void save( Record rec ) throws IOException
   {
      File file = m_config.getOutputFile();
      File parent = file.getParentFile();
      if (parent != null)
      {
         parent.mkdirs();
         if (!parent.isDirectory())
            throw new IOException( Resources.formatString( "msg.7012", parent.getAbsolutePath() ) ); //$NON-NLS-1$
      }
      try (FileOutputStream fos = new FileOutputStream( file, true ))
      {
         BufferedWriter output = new BufferedWriter( new OutputStreamWriter( fos, Constants.UTF_8 ) );
         m_writer.setOutput( output );
         m_writer.write( rec );
         output.newLine();
         output.flush();
      }
   }

   FileStore( Configuration config, RecordWriter writer )
   {
      m_config = config;
      m_writer = writer;
   }

   private Configuration m_config;
   private RecordWriter  m_writer;
}
