//
// Resources.java
//
// Copyright (c) 2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.apps.bot;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Utility class for retrieving externalized resources. This class delegates
 * to {@link com.dakengo.libs.application.Resources}.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public abstract class Resources
{
   public static final String TEXT_BUNDLE_NAME = "com.dakengo.apps.bot.text"; //$NON-NLS-1$

   public static String formatString( String key, Object... args )
   {
      return String.format( getString( key ), args );
   }

   public static String getString( String key )
   {
      String str;
      ResourceBundle bundle = ResourceBundle.getBundle( TEXT_BUNDLE_NAME );
      try
      {
         str = bundle.getString( key );
      }
      catch (MissingResourceException ex)
      {
         str = com.dakengo.libs.application.Resources.getString( key );
      }
      return str;
   }
}
