//
// BaseScorer.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import net.xeoh.plugins.base.annotations.Capabilities;
import net.xeoh.plugins.base.annotations.events.Init;
import net.xeoh.plugins.base.annotations.injections.InjectPlugin;
import net.xeoh.plugins.informationbroker.InformationBroker;
import net.xeoh.plugins.informationbroker.InformationItem;
import net.xeoh.plugins.informationbroker.util.InformationBrokerUtil;

/**
 * A base class for scoring plugins.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 * 
 * @see com.dakengo.libs.game.Scorer
 */
public abstract class BaseScorer
{
   /**
    * A plugin broker information item for publishing or subscribing
    * to the names of scoring systems.
    * 
    * @see net.xeoh.plugins.informationbroker.InformationBroker
    */
   public class NamesInfoItem implements InformationItem<Set<String>> {}

   /**
    * Returns a sorted set of names representing the scoring plugins
    * that have been loaded.
    */
   @SuppressWarnings("unchecked")    // I'd rather use @SuppressWarnings("varargs")
   public static SortedSet<String> getSystems()
   {
      if (m_broker == null)
         throw new IllegalStateException( Resources.getString( "msg.2028" ) ); //$NON-NLS-1$
      InformationBrokerUtil util = new InformationBrokerUtil( m_broker );
      return new TreeSet<String>( util.get( BaseScorer.NamesInfoItem.class, new HashSet<String>() ) );
   }

   /**
    * Publishes the name of this scorer's system.
    * 
    * @see net.xeoh.plugins.informationbroker.InformationBroker
    */
   @Init
   public void init()
   {
      PluginUtils.publishScoringSystem( m_broker, m_name );
   }

   /**
    * Returns the capabilities for this plugin.
    */
   @Capabilities
   public String[] getCapabilities()
   {
      return new String[] { Constants.SCORING_CAP_PREFIX + m_name };
   }

   protected BaseScorer( String name )
   {
      m_name = name;
   }

   @InjectPlugin
   public static InformationBroker m_broker;  // jspf injection requires public access specifier

   private String m_name = ""; //$NON-NLS-1$
}
