//
// ConfigPropertyValueProvider.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

/**
 * A value provider that returns a {@link Configuration} property value.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class ConfigPropertyValueProvider implements ConfigVariableValueProvider
{
   public ConfigPropertyValueProvider( Configuration config, String prop_name )
   {
      if (prop_name.isEmpty())
         throw new IllegalArgumentException( Resources.getString( "msg.2004" ) ); //$NON-NLS-1$
      m_prop_name = prop_name;
      m_config = config;
   }

   @Override
   public String getValue()
   {
      switch (m_prop_name)
      {
         case Configuration.DATE_FORMAT_VAR:
            return m_config.getDateFormat().toPattern();
         case Configuration.ENGINE_NAME_VAR:
            return m_config.getEngineName();
         case Configuration.ENGINE_VERSION_VAR:
            return m_config.getEngineVersion();
         case Configuration.FILENAME_FORMAT_VAR:
            return m_config.getFilenameFormat();
         case Configuration.KO_RULE_VAR:
            return m_config.getKoRule().toString();
         case Configuration.LAYERS_FILE_VAR:
            return m_config.getLayersFile().getAbsolutePath();
         case Configuration.OUTPUT_DIR_VAR:
            return m_config.getOutputDirectory().getAbsolutePath();
         case Configuration.SCORING_VAR:
            return m_config.getScoring().toString();
         case Configuration.SUICIDE_VAR:
            return Boolean.toString( m_config.allowSuicide() );
         default:
            throw new IllegalArgumentException( Resources.formatString( "msg.2005", m_prop_name ) ); //$NON-NLS-1$
      }
   }

   private Configuration m_config;
   private String        m_prop_name;
}
