//
// GameIdPatternConverter.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

import java.util.Map;

import org.apache.logging.log4j.ThreadContext;
import org.apache.logging.log4j.core.LogEvent;
import org.apache.logging.log4j.core.config.plugins.Plugin;
import org.apache.logging.log4j.core.pattern.ConverterKeys;
import org.apache.logging.log4j.core.pattern.LogEventPatternConverter;

/**
 * Implements the {@code %gid} conversion pattern for use when
 * specifying patterns in log4j 2.0 configuration files.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@Plugin(name = "gameID", type = "Converter")
@ConverterKeys({"gid"})
public class GameIdPatternConverter extends LogEventPatternConverter
{
   public static GameIdPatternConverter newInstance()
   {
      return new GameIdPatternConverter();
   }

   public GameIdPatternConverter()
   {
      super( "gameID", "" ); //$NON-NLS-1$ //$NON-NLS-2$
   }

   @Override
   public void format( LogEvent event, StringBuilder dest )
   {
      Map<String,String> ctx_map = ThreadContext.getContext();
      if (ctx_map.containsKey( Constants.GAME_ID_MDC_PROP ))
         dest.append( ctx_map.get( Constants.GAME_ID_MDC_PROP ) );
   }
}
