//
// GtpCommandHandler.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

import net.xeoh.plugins.base.Plugin;

/**
 * Implement this plugin interface to provide a GTP command. The implementation
 * must be annotated with the {@code @PluginImplementation} class annotation. The implementation
 * must be thread-safe.<p>
 * 
 * Each command plugin must publish its name via {@link PluginUtils#publishCommand}.<p>
 * 
 * Each plugin must implement a method that returns the plugin's capabilities,
 * identified via the {@code @Capabilities}
 * annotation. The required capability is a single string having the prefix
 * {@link Constants#GTP_CAP_PREFIX}
 * and a suffix consisting of a command name.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 * 
 * @see BaseGtpCommandHandler
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/base/annotations/Capabilities.html"><code>@Capabilities</code></a>
 * @see GtpEngine
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/informationbroker/InformationBroker.html"><code>InformationBroker</code></a>
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/base/annotations/events/Init.html"><code>@Init</code></a>
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/base/annotations/PluginImplementation.html"><code>@PluginImplementation</code></a>
 * @see <a href="http://www.lysator.liu.se/~gunnar/gtp/gtp2-spec-draft2/gtp2-spec.html">Go Text Protocol</a>
 */
public interface GtpCommandHandler extends Plugin
{
   /**
    * This method implements the behavior of the command.
    * 
    * @param cmd
    * @param context
    * @param callback
    */
   void handleCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback );

   /**
    * Return true if the current game record should be saved
    * prior to execution of this command.
    */
   boolean shouldSave();
}
