//
// PluginException.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

/**
 * An exception class representing an error encountered when initializing 
 * a plugin.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@SuppressWarnings( "serial" )
public class PluginException extends RuntimeException
{
   public PluginException()
   {
   }

   public PluginException( String details )
   {
      super( details );
      m_details = details;
   }

   public PluginException( Throwable cause )
   {
      super( cause );
   }

   public PluginException( String details, Throwable cause )
   {
      super( details, cause );
      m_details = details;
   }

   public PluginException( String details, String plugin_id )
   {
      super( String.format( "%s: %s", details, plugin_id ) ); //$NON-NLS-1$
      m_details = details;
      m_plugin_id = plugin_id;
   }

   public PluginException( String details, String plugin_id, Throwable cause )
   {
      super( String.format( "%s: %s", details, plugin_id ), cause ); //$NON-NLS-1$
      m_details = details;
      m_plugin_id = plugin_id;
   }

   public String getDetails()
   {
      return m_details;
   }

   public String getPluginID()
   {
      return m_plugin_id;
   }

   private String m_details   = ""; //$NON-NLS-1$
   private String m_plugin_id = ""; //$NON-NLS-1$
}
