//
// SystemPropertyValueProvider.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.application;

/**
 * A value provider that returns the value of a system property.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class SystemPropertyValueProvider implements ConfigVariableValueProvider
{
   /**
    * @param prop_name  system property to be retrieved
    */
   public SystemPropertyValueProvider( String prop_name )
   {
      if (prop_name.isEmpty())
         throw new IllegalArgumentException( Resources.getString( "msg.2026" ) ); //$NON-NLS-1$
      m_prop_name = prop_name;
   }

   @Override
   public String getValue()
   {
      return System.getProperty( m_prop_name );
   }

   private String m_prop_name;
}
