//
// GameException.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.game;

/**
 * An exception class representing a rule violation, such as a player attempting
 * to move out of turn.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@SuppressWarnings( "serial" )
public class GameException extends RuntimeException
{
   public GameException()
   {
   }

   /**
    * @param details
    */
   public GameException( String details )
   {
      super( details );
   }

   /**
    * The specified point must be valid for the current board size.
    * 
    * @param details
    * @param point
    */
   public GameException( String details, short point )
   {
      super( details + ": " + Convert.pointToString( point ) ); //$NON-NLS-1$
      m_point = point;
   }

   /**
    * The specified color can be any value.
    * 
    * @param details
    * @param color
    */
   public GameException( String details, Colors color )
   {
      super( details + ": " + color ); //$NON-NLS-1$
      m_color = color;
   }

   /**
    * The specified point must be a valid point for the current
    * board size.
    * 
    * @param details
    * @param color
    * @param point
    */
   public GameException( String details, Colors color, short point )
   {
      super( details + ": " + color + ": " + Convert.pointToString( point ) ); //$NON-NLS-1$ //$NON-NLS-2$
      m_color = color;
      m_point = point;
   }

   public Colors getColor()
   {
      return m_color;
   }

   public short getPoint()
   {
      return m_point;
   }

   private Colors m_color = Colors.DONT_CARE;
   private short  m_point = Board.INVALID_POINT;
}
