//
// MoveTypes.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.game;

/**
 * A classification of moves.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public enum MoveTypes
{
   /** The move violates the current ko rule. */
   KO,
   /** The move is legal. */
   LEGAL,
   /** The move is illegal because the specified point is already occupied. */
   OCCUPIED,
   /** The move is legal and places the opponent in atari. */
   OPPONENT_ATARI,
   /** The move is illegal because the player played out of turn. */
   OUT_OF_TURN,
   /** The player passed. */
   PASS,
   /** The player resigned the game. */
   RESIGN,
   /** The move is legal, but the player is putting itself into atari. */
   SELF_ATARI,
   /** If suicide moves are disallowed, then the move is illegal because it is a suicide move. */
   SUICIDE
}
