//
// Playout.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.player;

import org.apache.commons.lang3.tuple.ImmutablePair;

import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;

/**
 * Estimate the value of a board position by running simulated games.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public interface Playout
{
   /**
    * @return Current komi value.
    */
   double getKomi();

   /**
    * @param komi  new komi value
    */
   void setKomi( double komi );

   /**
    * Based on a starting board position, play a series of moves until no further
    * moves are possible, then score the resulting board position. Do this a specified
    * number of times to estimate the value of the starting board position.<p>
    * 
    * Each playout should be played to the bitter end. Suicide moves should be disallowed.
    * 
    * @param board      starting position
    * @param test_pnt   candidate move to be evaluated
    * @param to_play    player to move
    * @param count      number of playouts to run
    * 
    * @return  {@code ImmutablePair} in which the first element is the score representing the value of the move
    *          and the second element is the time in millis that was required to run the specified number
    *          of playouts. Positive scores are good for the specified player to move, negative scores are bad.
    */
   ImmutablePair<Integer,Long> run( Board board, short test_pnt, Colors to_play, int count );
}
