//
// TreePolicy.java
//
// Copyright (c) 2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.player;

/**
 * Implement this interface to define the policy for selecting and expanding
 * child nodes.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public interface TreePolicy<T extends BaseTreeNode>
{
   /**
    * Determine whether the specified node should be expanded.
    * 
    * @param cursor
    * 
    * @return  {@code true} if the specified node should be expanded according to
    *          this tree policy; {@code false} otherwise.
    */
   boolean shouldExpand( TreeCursor<T> cursor );

   /**
    * Evaluate the specified node's children to choose one for further exploration.
    * 
    * @param cursor
    * 
    * @return The index of the selected child.
    */
   int selectChild( TreeCursor<T> cursor );
}
