//
// DiagramSettings.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import com.dakengo.libs.base.HashUtils;

/**
 * A class for configuring diagram display behavior.
 * 
 * <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class DiagramSettings
{
   public DiagramSettings()
   {
   }

   public DiagramSettings( DiagramSettings other )
   {
      m_coords = other.m_coords;
      m_hoshi_dots = other.m_hoshi_dots;
      m_moves_not_shown = other.m_moves_not_shown;
      m_name = other.m_name;
      m_remove_captured = other.m_remove_captured;
      m_app_defaults = other.m_app_defaults;
   }

   public DiagramSettings( boolean enable_defaults, boolean enable_coords, boolean enable_hoshi, boolean enable_moves_not_shown, boolean enable_name, boolean enable_remove_captured )
   {
      m_app_defaults = enable_defaults;
      m_coords = enable_coords;
      m_hoshi_dots = enable_hoshi;
      m_moves_not_shown = enable_moves_not_shown;
      m_name = enable_name;
      m_remove_captured = enable_remove_captured;
   }

   public boolean coordsEnabled()
   {
      return m_coords;
   }

   public void enableCoords( boolean enabled )
   {
      m_coords = enabled;
   }

   public boolean hoshiDotsEnabled()
   {
      return m_hoshi_dots;
   }

   public void enableHoshiDots( boolean enabled )
   {
      m_hoshi_dots = enabled;
   }

   public boolean movesNotShownEnabled()
   {
      return m_moves_not_shown;
   }

   public void enableMovesNotShown( boolean enabled )
   {
      m_moves_not_shown = enabled;
   }

   public boolean nameEnabled()
   {
      return m_name;
   }

   public void enableName( boolean enabled )
   {
      m_name = enabled;
   }

   public boolean removeCapturedEnabled()
   {
      return m_remove_captured;
   }

   public void enableRemoveCaptured( boolean enabled )
   {
      m_remove_captured = enabled;
   }

   public boolean appDefaultsEnabled()
   {
      return m_app_defaults;
   }

   public void enableAppDefaults( boolean enabled )
   {
      m_app_defaults = enabled;
   }

   @Override
   public boolean equals( Object obj )
   {
      if (!(obj instanceof DiagramSettings))
         return false;
      if (obj == this)
         return true;
      DiagramSettings other = (DiagramSettings) obj;
      return m_coords          == other.m_coords          &&
             m_hoshi_dots      == other.m_hoshi_dots      &&
             m_moves_not_shown == other.m_moves_not_shown &&
             m_name            == other.m_name            &&
             m_remove_captured == other.m_remove_captured &&
             m_app_defaults    == other.m_app_defaults;
   }

   @Override
   public int hashCode()
   {
      return HashUtils.compute( m_coords, m_hoshi_dots, m_moves_not_shown, m_name, m_remove_captured, m_app_defaults );
   }

   @Override
   public String toString()
   {
      StringBuilder buffer = new StringBuilder();
      buffer.append( m_coords );          buffer.append( "," ); //$NON-NLS-1$
      buffer.append( m_hoshi_dots );      buffer.append( "," ); //$NON-NLS-1$
      buffer.append( m_moves_not_shown ); buffer.append( "," ); //$NON-NLS-1$
      buffer.append( m_name );            buffer.append( "," ); //$NON-NLS-1$
      buffer.append( m_remove_captured ); buffer.append( "," ); //$NON-NLS-1$
      buffer.append( m_app_defaults );
      return buffer.toString();
   }

   private boolean m_coords          = false;
   private boolean m_hoshi_dots      = false;
   private boolean m_moves_not_shown = false;
   private boolean m_name            = false;
   private boolean m_remove_captured = false;
   private boolean m_app_defaults    = true;
}
