//
// Handicaps.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

/**
 * Specifies the placement method for handicap stones. The
 * value {@link #FIXED} indicates that stones are placed in standard locations
 * which depend on the board size.
 * The {@link #FREE} value makes the alternative choice explicit, whereas
 * the {@link #UNKNOWN} value is used for circumstances such as parsing
 * an SGF record, where neither of the other two choices are identified.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 * 
 * @see <a href="http://senseis.xmp.net/?Handicap">Handicap</a>
 */
public enum Handicaps
{
   /** Traditional placement of handicap stones on the star points. */
   FIXED,
   /** Unrestricted placement of handicap stones. */
   FREE,
   /** No handicap stones. */
   NONE,
   /** The handicap placement policy is unspecified. */
   UNKNOWN
}
