//
// PrintSettings.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import com.dakengo.libs.base.HashUtils;

/**
 * A class for configuring record printing behavior.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class PrintSettings
{
   private static final int DEFAULT_MOVE_NUMBER_MODULO = 100;

   public enum MoveNumbers
   {
      DISABLE,
      AS_IS,
      MODULO
   }

   public PrintSettings()
   {
   }

   public PrintSettings( PrintSettings other )
   {
      m_move_numbers = other.m_move_numbers;
      m_move_num_modulo = other.m_move_num_modulo;
   }

   public int getMoveNumberModulo()
   {
      return m_move_num_modulo;
   }

   public void setMoveNumberModulo( int modulo )
   {
      if (modulo <= 0)
         throw new IllegalArgumentException( Resources.formatString( "msg.4012", modulo ) ); //$NON-NLS-1$
      m_move_num_modulo = modulo;
      m_move_numbers = MoveNumbers.MODULO;
   }

   public MoveNumbers getMoveNumbers()
   {
      return m_move_numbers;
   }

   public void setMoveNumbers( MoveNumbers setting )
   {
      m_move_numbers = setting;
      if (m_move_numbers == MoveNumbers.MODULO)
         m_move_num_modulo = DEFAULT_MOVE_NUMBER_MODULO;
      else
         m_move_num_modulo = 0;
   }

   @Override
   public boolean equals( Object obj )
   {
      if (!(obj instanceof PrintSettings))
         return false;
      if (obj == this)
         return true;
      PrintSettings other = (PrintSettings) obj;
      if (m_move_numbers != other.m_move_numbers)
         return false;
      return m_move_numbers != MoveNumbers.MODULO || m_move_num_modulo == other.m_move_num_modulo;
   }

   @Override
   public int hashCode()
   {
      return HashUtils.compute( m_move_numbers, m_move_num_modulo );
   }

   @Override
   public String toString()
   {
      return String.format( "%s,%d", m_move_numbers.toString(), m_move_num_modulo ); //$NON-NLS-1$
   }

   private MoveNumbers m_move_numbers    = MoveNumbers.AS_IS;
   private int         m_move_num_modulo = 0;
}
