//
// PropertyException.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

/**
 * An exception class represent an invalid property detected in a game record.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@SuppressWarnings( "serial" )
public class PropertyException extends RuntimeException
{
   public static String getActualClass( ClassCastException ex )
   {
      String msg = ex.getMessage();
      int index = msg.indexOf( " cannot be cast to" );  // FIXME: probably locale-specific
      if (index < 0)
         return Resources.getString( "msg.4013" ); //$NON-NLS-1$
      return msg.substring( 0, index );
   }

   public static String getExpectedClass( ClassCastException ex )
   {
      String token = "cannot be cast to ";  // FIXME: probably locale-specific
      String msg = ex.getMessage();
      int index = msg.indexOf( token );
      if (index < 0)
         return Resources.getString( "msg.4013" ); //$NON-NLS-1$
      return msg.substring( index + token.length(), msg.length() );
   }

   public PropertyException( int prop_id )
   {
      super( Resources.formatString( "msg.4006", prop_id ) ); //$NON-NLS-1$
      m_id = prop_id;
   }

   public PropertyException( String msg )
   {
      super( msg );
   }

   public PropertyException( int prop_id, ClassCastException ex )
   {
      super( Resources.formatString( "msg.4014", PropertyID.toString( prop_id ), getExpectedClass( ex ), getActualClass( ex ) ) ); //$NON-NLS-1$
      m_id = prop_id;
   }

   public PropertyException( int prop_id, String actual_class, String expected_class )
   {
      super( Resources.formatString( "msg.4014", PropertyID.toString( prop_id ), expected_class, actual_class ) ); //$NON-NLS-1$
      m_id = prop_id;
   }

   public PropertyException( int prop_id, String value_text )
   {
      super( Resources.formatString( "msg.4015", PropertyID.toString( prop_id ), value_text ) ); //$NON-NLS-1$
      m_id = prop_id;
      m_value = value_text;
   }

   public int getID()
   {
      return m_id;
   }

   public String getValue()
   {
      return m_value;
   }

   private int    m_id     = Integer.MAX_VALUE;
   private String m_value  = ""; //$NON-NLS-1$
}
