//
// PropertyVisitor.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

/**
 * Implement this interface to process the content of a game record.
 *
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 *
 * @see GameProperties
 * @see PositionProperties
 * @see PropertyID
 * @see Record#visit
 */
public interface PropertyVisitor
{
   /**
    * Implement this method to be notified when a new branch is about to
    * be visited. Return true if the branch should be traversed before
    * proceeding to the next entry on the current branch, or false
    * to skip the sub-branch.
    */
   boolean startBranch( PositionProperties branch );

   /**
    * Implement this method to be notified when a branch is completed.
    */
   void finishBranch( PositionProperties branch );

   /**
    * Implement this method to process a property.
    */
   void accept( int id, Object value );
}
