//
// RecordWriter.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import net.xeoh.plugins.base.Plugin;

import java.io.IOException;
import java.io.Writer;

/**
 * Plugins implement this interface to provide an output writer for a game record format.
 * The implementation must be annotated with the {@code @PluginImplementation} class annotation.
 * The implementation must be thread-safe.<p>
 * 
 * Each plugin must implement a method that returns the plugin's capabilities,
 * identified via the {@code @Capabilities}
 * annotation. Two capability strings are required for {@code RecordWriter}.
 * One is a string whose prefix is
 * {@link com.dakengo.libs.application.Constants#MODE_CAP_PREFIX}
 * and whose suffix is {@link com.dakengo.libs.application.Constants#WRITER_CAP_SUFFIX}.
 * Another is a string whose prefix is
 * {@link com.dakengo.libs.application.Constants#FORMAT_CAP_PREFIX}
 * and whose suffix is the format's file extension without any leading dot (e.g., "sgf").
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 * 
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/base/annotations/Capabilities.html"><code>@Capabilities</code></a>
 * @see <a href="http://data.xeoh.net/jspf/api/net/xeoh/plugins/base/annotations/PluginImplementation.html"><code>@PluginImplementation</code></a>
 */
public interface RecordWriter extends Plugin
{
   /**
    * Sets the writer's output.
    * 
    * @param writer
    */
   void setOutput( Writer writer );

   /**
    * Writes the specified game record to the output.
    */
   void write( Record rec ) throws IOException;
}
