//
// RenderingStyle.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import com.dakengo.libs.base.HashUtils;

/**
 * A value class that provides hints for rendering game records. The
 * initial set of hints implemented by this class is heavily influenced
 * by the SGF format.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class RenderingStyle
{
   public RenderingStyle()
   {
   }

   public RenderingStyle( boolean show_siblings, boolean show_markup )
   {
      m_siblings = show_siblings;
      m_markup = show_markup;
   }

   public boolean showMarkup()
   {
      return m_markup;
   }

   public boolean showSiblings()
   {
      return m_siblings;
   }

   @Override
   public boolean equals( Object obj )
   {
      if (!(obj instanceof RenderingStyle))
         return false;
      if (obj == this)
         return true;
      RenderingStyle other = (RenderingStyle) obj;
      return m_markup == other.m_markup && m_siblings == other.m_siblings;
   }

   @Override
   public int hashCode()
   {
      return HashUtils.compute( m_markup, m_siblings );
   }

   @Override
   public String toString()
   {
      return String.format( "%b,%b", m_markup, m_siblings ); //$NON-NLS-1$
   }

   private boolean m_markup   = true;
   private boolean m_siblings = false;
}
