//
// Round.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import com.dakengo.libs.base.HashUtils;

/**
 * This value class represents a competition level of an event in which a game
 * is played.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public class Round
{
   public enum Type
   {
      UNDEFINED, FINAL, GAME, PLAYOFF, PRELIM, LEAGUE
   }

   public Round()
   {
   }

   public Round( int number, Type type, String extra )
   {
      if (m_number < 0)
         throw new IllegalArgumentException( Resources.getString( "msg.4018" ) ); //$NON-NLS-1$
      m_number = number;
      m_type = type;
      m_extra = extra;
   }

   public String getExtra()
   {
      return m_extra;
   }

   public int getNumber()
   {
      return m_number;
   }

   public Type getType()
   {
      return m_type;
   }

   public boolean equals( Object obj )
   {
      if (!(obj instanceof Round))
         return false;
      if (this == obj)
         return true;
      Round other = (Round) obj;
      return m_type == other.m_type && m_number == other.m_number && m_extra.equals( other.m_extra );
   }

   public int hashCode()
   {
      return HashUtils.compute( m_type, m_number, m_extra );
   }

   public String toString()
   {
      return String.format( "%s,%d,'%s'", m_type, m_number, m_extra ); //$NON-NLS-1$
   }

   private String m_extra  = ""; //$NON-NLS-1$
   private int    m_number = 1;
   private Type   m_type   = Type.GAME;
}
