//
// SignatureBuilder.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.libs.record;

import org.apache.commons.lang3.tuple.ImmutablePair;

import com.dakengo.libs.game.Colors;
import com.dakengo.libs.game.Signature;

/**
 * This is a helper class for computing a game signature from a {@link Record}.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
public abstract class SignatureBuilder
{
   /**
    * Computes and returns a game signature.
    * 
    * @param record
    */
   public static Signature compute( Record record )
   {
      final Signature sig = new Signature();
      record.getMainBranch().visit( new PropertyVisitor()
      {
         @Override
         public boolean startBranch( PositionProperties branch )
         {
            return true;
         }

         @Override
         public void finishBranch( PositionProperties branch )
         {
         }

         @SuppressWarnings( "unchecked" )
         @Override
         public void accept( int id, Object value )
         {
            if (id != PropertyID.MOVE)
               return;
            ImmutablePair<Colors,Short> move = (ImmutablePair<Colors,Short>) value;
            if (Signature.isTurnNeeded( m_turn ))
               sig.addPoint( move.getRight() );
            m_turn++;
         }

         private int m_turn = 0;
      },
      true );
      return sig;
   }

   private SignatureBuilder()
   {
   }
}
