//
// BoardSizeHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.io.IOException;
import java.util.List;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;
import com.dakengo.libs.game.Board;

/** 
 * Implements the {@code boardsize} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class BoardSizeHandler extends BaseGtpCommandHandler
{
   public BoardSizeHandler()
   {
      super( "boardsize", true ); //$NON-NLS-1$
   }

   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      List<String> args = cmd.getArguments();
      if (args.size() != 1)
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      int size = -1;
      try
      {
         size = Integer.parseInt( args.get( 0 ) );
      }
      catch (NumberFormatException ex)
      {
         callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2044" ) ); //$NON-NLS-1$
         logError( Resources.getString( "msg.3013" ) ); //$NON-NLS-1$
         return;
      }
      if (!Board.isValidSize( size ))
      {
         callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2030" ) ); //$NON-NLS-1$
         logError( Resources.getString( "msg.3006" ) ); //$NON-NLS-1$
         return;
      }
      if (Board.getSize() != size)
      {
         logInfo( Resources.formatString( "msg.3009", size ) ); //$NON-NLS-1$
         try
         {
            context.loadLayers( size );
         }
         catch (IOException ex)
         {
            callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2037" ) ); //$NON-NLS-1$
            logThrowable( ex );
            return;
         }
      }
      context.reset();
      callback.handleReply( false, cmd.getID(), "" ); //$NON-NLS-1$
   }
}
