//
// KnownCommandHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import net.xeoh.plugins.base.annotations.PluginImplementation;
import net.xeoh.plugins.informationbroker.util.InformationBrokerUtil;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;

/** 
 * Implements the {@code known_command} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class KnownCommandHandler extends BaseGtpCommandHandler
{
   public KnownCommandHandler()
   {
      super( "known_command", false ); //$NON-NLS-1$
   }

   @SuppressWarnings("unchecked")    // I'd rather use @SuppressWarnings("varargs")
   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      List<String> args = cmd.getArguments();
      if (args.size() != 1)
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      String name = args.get( 0 );
      InformationBrokerUtil util = new InformationBrokerUtil( m_broker );
      Set<String> names = util.get( GtpCommand.NamesInfoItem.class, new HashSet<String>() );
      callback.handleReply( false, cmd.getID(), names.contains( name ) ? Resources.getString( "msg.2045" ) : Resources.getString( "msg.2038" ) ); //$NON-NLS-1$ //$NON-NLS-2$
   }
}
