//
// KomiHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.math.BigDecimal;
import java.util.List;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;

/** 
 * Implements the {@code komi} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class KomiHandler extends BaseGtpCommandHandler
{
   public KomiHandler()
   {
      super( "komi", false ); //$NON-NLS-1$
   }

   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      List<String> args = cmd.getArguments();
      if (args.size() != 1)
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      BigDecimal komi = BigDecimal.ZERO;
      try
      {
         komi = new BigDecimal( args.get( 0 ) );
      }
      catch (NumberFormatException ex)
      {
         callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2044" ) ); //$NON-NLS-1$
         logError( Resources.getString( "msg.3007" ) ); //$NON-NLS-1$
         return;
      }
      context.reset();
      if (context.getGame().getKomi() != komi)
      {
         logInfo( Resources.formatString( "msg.3012", komi.toPlainString() ) ); //$NON-NLS-1$
         context.getGame().setKomi( komi );
      }
      callback.handleReply( false, cmd.getID(), "" ); //$NON-NLS-1$
   }
}
