//
// ListCommandsHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import net.xeoh.plugins.base.annotations.PluginImplementation;
import net.xeoh.plugins.informationbroker.util.InformationBrokerUtil;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;

/** 
 * Implements the {@code list_commands} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class ListCommandsHandler extends BaseGtpCommandHandler
{
   public ListCommandsHandler()
   {
      super( "list_commands", false ); //$NON-NLS-1$
   }

   @SuppressWarnings("unchecked")    // I'd rather use @SuppressWarnings("varargs")
   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      if (!cmd.getArguments().isEmpty())
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      SortedSet<String> sorted_names = new TreeSet<String>();
      InformationBrokerUtil util = new InformationBrokerUtil( m_broker );
      Set<String> names = util.get( GtpCommand.NamesInfoItem.class, new HashSet<String>() );
      for (String name : names)
         sorted_names.add( name );
      String eol = System.getProperty( "line.separator" ); //$NON-NLS-1$
      StringBuilder buffer = new StringBuilder();
      int count = 0;
      for (String name : sorted_names)
      {
         buffer.append( name );
         count++;
         if (count < names.size())
            buffer.append( eol );
      }
      callback.handleReply( false, cmd.getID(), buffer.toString() );
   }
}
