//
// PlayHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.util.List;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;
import com.dakengo.libs.base.ParseException;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.game.Convert;
import com.dakengo.libs.game.Game;
import com.dakengo.libs.record.PositionProperties;

/** 
 * Implements the {@code play} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class PlayHandler extends BaseGtpCommandHandler
{
   public PlayHandler()
   {
      super( "play", false ); //$NON-NLS-1$
   }

   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      List<String> args = cmd.getArguments();
      if (args.size() != 2)
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      Colors color = Colors.DONT_CARE;
      short pnt = Board.INVALID_POINT;
      setPlayGame( cmd, context, callback );  // also sets board size, which we need to correctly parse the point argument
      try
      {
         color = Convert.stringToColor( args.get( 0 ) );
         if (!Game.isPlayerColor( color ))
         {
            callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2044" ) ); //$NON-NLS-1$
            logError( Resources.getString( "msg.3003" ) ); //$NON-NLS-1$
         }
         String text = args.get( 1 ).toLowerCase();
         if (!text.equals( "pass" )) //$NON-NLS-1$
            pnt = Convert.stringToPoint( text );
      }
      catch (ParseException ex)
      {
         callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2044" ) ); //$NON-NLS-1$
         logError( ex.getMessage() );
         return;
      }
      context.setOpponentName( color );
      context.getGame().play( pnt, null );
      PositionProperties branch = context.getRecord().getMainBranch();
      branch.addMove( color, pnt );
      branch.addNote( 0, "NODE", "" ); //$NON-NLS-1$ //$NON-NLS-2$ // FIXME: node numbers
      callback.handleReply( false, cmd.getID(), "" ); //$NON-NLS-1$
   }
}
