//
// SetFreeHandicapHandler.java
//
// Copyright (c) 2012-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.core;

import java.util.HashSet;
import java.util.Set;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;
import com.dakengo.libs.base.ParseException;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.game.Convert;
import com.dakengo.libs.game.Game;
import com.dakengo.libs.record.PositionProperties;

/** 
 * Implements the {@code set_free_handicap} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class SetFreeHandicapHandler extends BaseGtpCommandHandler
{
   public SetFreeHandicapHandler()
   {
      super( "set_free_handicap", false ); //$NON-NLS-1$
   }

   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      int arg_count = cmd.getArguments().size(); 
      if (arg_count < 2 || arg_count >= Board.getTotal() - 1)
      {
         callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2031" ) ); //$NON-NLS-1$
         logError( Resources.getString( "msg.2040" ) ); //$NON-NLS-1$
         return;
      }
      Set<Short> pnts = new HashSet<Short>();
      for (String arg : cmd.getArguments())
      {
         short pnt = Board.INVALID_POINT;
         try
         {
            pnt = Convert.stringToPoint( arg );
         }
         catch (ParseException ex)
         {
            callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2044" ) ); //$NON-NLS-1$
            logError( ex.getMessage() );
            return;
         }
         if (pnts.contains( pnt ))
         {
            callback.handleReply( true, cmd.getID(), Resources.getString( "msg.2031" ) ); //$NON-NLS-1$
            logError( Resources.getString( "msg.3002" ) ); //$NON-NLS-1$
            return;
         }
         pnts.add( pnt );
      }
      setPlayGame( cmd, context, callback );
      Game game = context.getGame();
      if (game.getBoard().getCount( Colors.EMPTY ) != Board.getTotal())
      {
         String msg = Resources.getString( "msg.2033" ); //$NON-NLS-1$
         callback.handleReply( true, cmd.getID(), msg );
         logError( msg );
         return;
      }
      game.placeStones( Colors.BLACK, pnts );
      game.setPlayer( Colors.WHITE );
      PositionProperties branch = context.getRecord().getMainBranch();
      branch.addStones( Colors.BLACK, pnts );
      callback.handleReply( false, cmd.getID(), "" ); //$NON-NLS-1$
      logInfo( Resources.getString( "msg.3011" ) ); //$NON-NLS-1$
   }
}
