//
// ShowBoardHandler.java
//
// Copyright (c) 2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.gtp.debug;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import com.dakengo.libs.application.BaseGtpCommandHandler;
import com.dakengo.libs.application.GtpCommand;
import com.dakengo.libs.application.GameContext;
import com.dakengo.libs.application.GtpReplyHandler;
import com.dakengo.libs.application.TextUtils;

/** 
 * Implements the {@code showboard} GTP command.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
@PluginImplementation
public class ShowBoardHandler extends BaseGtpCommandHandler
{
   public ShowBoardHandler()
   {
      super( "showboard", false ); //$NON-NLS-1$
   }

   @Override
   protected void doCommand( GtpCommand cmd, GameContext context, GtpReplyHandler callback )
   {
      if (!cmd.getArguments().isEmpty())
      {
         reportInvalidNumArgs( cmd, callback );
         return;
      }
      StringBuilder buffer = new StringBuilder();
      buffer.append( System.getProperty( "line.separator" ) ); //$NON-NLS-1$
      buffer.append( TextUtils.print( context.getGame().getBoard() ) );
      callback.handleReply( false, cmd.getID(), buffer.toString() );
   }
}
