//
// MoveFilter.java
//
// Copyright (c) 2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.player.sample;

import java.util.List;

import org.apache.commons.lang3.tuple.ImmutablePair;

import com.dakengo.libs.base.MersenneTwisterFast;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.player.BaseLegalMoveFilter;

// TODO: consider moving this class to com.dakengo.libs.player

/**
 * This move filter just randomly shuffles the candidates.
 * 
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
class RandomMoveFilter extends BaseLegalMoveFilter
{
   /**
    * @param prng  random number generator, assumed to be thread-local
    */
   RandomMoveFilter( MersenneTwisterFast prng )
   {
      m_prng = prng;
   }

   @Override
   protected void sort( Colors player, List<ImmutablePair<Short, Board>> candidates )
   {
      int count = candidates.size();
      if (count <= 1)
         return;

      // Unlike a real move generator, the random player doesn't need to evaluate all of the
      // possible moves before ranking them. Therefore, we just swap element 0 with a random
      // element.

      int i = m_prng.nextInt( count );  // nextInt() returns an integer drawn uniformly from 0 to n-1
      ImmutablePair<Short, Board> tmp = candidates.get( 0 );
      candidates.set( 0, candidates.get( i ) );
      candidates.set( i, tmp );
   }

   private MersenneTwisterFast m_prng;
}
