//
// AreaScorer.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.scoring.area;

import net.xeoh.plugins.base.annotations.PluginImplementation;
import org.apache.commons.lang3.tuple.ImmutablePair;

import com.dakengo.libs.application.BaseScorer;
import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.Colors;
import com.dakengo.libs.game.Scorer;

/**
 * Implements the area scoring system. Assumes that either a) dead stones have
 * already been removed or b) the game has been played to the end.
 *
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 * 
 * @see <a href="http://senseis.xmp.net/?Scoring">Scoring</a>
 */
@PluginImplementation
public class AreaScorer extends BaseScorer implements Scorer
{
   public AreaScorer()
   {
      super( "area" ); //$NON-NLS-1$
   }

   @Override
   public ImmutablePair<Integer,Integer> computeScores( Board board, short[] available )
   {
      ScoringVisitor visitor = new ScoringVisitor();
      board.visitPoints( Colors.DONT_CARE, visitor );
      if (available != null)
         visitor.getAvailable( available );
      return ImmutablePair.of( visitor.getBlackScore(), visitor.getWhiteScore() );
   }
}
