//
// ScoringVisitor.java
//
// Copyright (c) 2010-2013, Jack D. Unrue
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package com.dakengo.plugins.scoring.area;

import java.util.Deque;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;

import com.dakengo.libs.game.Board;
import com.dakengo.libs.game.PointVisitor;
import com.dakengo.libs.game.Colors;

/**
 * @author <a href="mailto:jdunrue@gmail.com">Jack D. Unrue</a>
 */
class ScoringVisitor implements PointVisitor
{
   public int getBlackScore()
   {
      return m_black_score;
   }

   public int getWhiteScore()
   {
      return m_white_score;
   }

   public int getAvailable( short[] buffer )
   {
      int count = m_history.size();
      if (count > buffer.length)
         throw new IllegalArgumentException( Resources.getString( "msg.6000" ) ); //$NON-NLS-1$
      int i = 0;
      for (short pnt : m_history)
         buffer[i++] = pnt;
      for ( ; i < buffer.length; i++)
         buffer[i] = Board.INVALID_POINT;
      return count;
   }

   @Override
   public boolean accept( Board board, short point )
   {
      if (m_history.contains( point ))
         return true;
      switch( board.getColor( point ) )
      {
         case BLACK:
            m_black_score++;
            return true;
         case WHITE:
            m_white_score++;
            return true;
         case EMPTY:
            break;
         default:
            throw new IllegalStateException( Resources.getString( "msg.6001" ) ); //$NON-NLS-1$
      }
      m_saw_black = false;
      m_saw_white = false;
      int prior_count = m_history.size();
      m_point_q.clear();
      m_point_q.addLast( point );
      while (m_point_q.size() > 0)
      {
         point = m_point_q.removeFirst();
         if (m_history.contains( point ))
            continue;
         m_history.add( point );
         board.visitNeighbors( point, m_helper );
      }
      int new_count = m_history.size() - prior_count;
      if (m_saw_black && !m_saw_white)
         m_black_score += new_count;
      else if (!m_saw_black && m_saw_white)
         m_white_score += new_count;
      return true;
   }

   private PointVisitor m_helper = new PointVisitor()
   {
      @Override
      public boolean accept( Board board, short point )
      {
         Colors color = board.getColor( point );
         if (color == Colors.BLACK)
            m_saw_black = true;
         else if (color == Colors.WHITE)
            m_saw_white = true;
         else
            m_point_q.addLast( point );
         return true;
      }
   };

   private boolean      m_saw_black   = false;
   private boolean      m_saw_white   = false;
   private int          m_black_score = 0;
   private int          m_white_score = 0;
   private Set<Short>   m_history     = new HashSet<Short>();
   private Deque<Short> m_point_q     = new LinkedList<Short>();
}
